@@ -11,19 +11,21 @@
 
 ##########NOTE: Does not conform to new interfaces yet #################
 
-from wcsBase import WCSBase, WCSCapabilitiesReader, RereadableURL
+from wcsBase import WCSBase, WCSCapabilitiesReader, ServiceException
+from owslib.util import openURL, testXMLValue
 from urllib import urlencode
 from urllib2 import urlopen
 from owslib.etree import etree
 import os, errno
 from owslib.coverage import wcsdecoder
+from owslib.crs import Crs
+
+import logging
+from owslib.util import log
 
 def ns(tag):
     return '{http://www.opengis.net/wcs/1.1}'+tag
 
-class ServiceException(Exception):
-    pass
-
 class WebCoverageService_1_1_0(WCSBase):
     """Abstraction for OGC Web Coverage Service (WCS), version 1.1.0
     Implements IWebCoverageService.
@@ -36,24 +38,34 @@ class WebCoverageService_1_1_0(WCSBase):
         else:
             raise KeyError, "No content named %s" % name
     
-    def __init__(self,url,xml):
+    def __init__(self,url,xml, cookies):
         self.version='1.1.0'
         self.url = url   
+        self.cookies=cookies
         # initialize from saved capability document or access the server
         reader = WCSCapabilitiesReader(self.version)
         if xml:
             self._capabilities = reader.readString(xml)
         else:
             self._capabilities = reader.read(self.url)
-            
+
+        # check for exceptions
+        se = self._capabilities.find('{http://www.opengis.net/ows/1.1}Exception')
+
+        if se is not None:
+            err_message = str(se.text).strip()
+            raise ServiceException(err_message, xml)
+
         #build metadata objects:
         
         #serviceIdentification metadata
         elem=self._capabilities.find('{http://www.opengis.net/wcs/1.1/ows}ServiceIdentification')
+        if elem is None:
+            elem=self._capabilities.find('{http://www.opengis.net/ows/1.1}ServiceIdentification')
         self.identification=ServiceIdentification(elem)
         
         #serviceProvider
-        elem=self._capabilities.find('{http://www.opengis.net/ows}ServiceProvider')
+        elem=self._capabilities.find('{http://www.opengis.net/ows/1.1}ServiceProvider')
         self.provider=ServiceProvider(elem)
                 
         #serviceOperations
@@ -115,15 +127,16 @@ class WebCoverageService_1_1_0(WCSBase):
         note: additional **kwargs helps with multi-version implementation
         core keyword arguments should be supported cross version
         example:
-        cvg=wcs.getCoverageRequest(identifier=['TuMYrRQ4'], timeSequence=['2792-06-01T00:00:00.0'], bbox=(-112,36,-106,41),format='application/netcdf', store='true')
+        cvg=wcs.getCoverageRequest(identifier=['TuMYrRQ4'], time=['2792-06-01T00:00:00.0'], bbox=(-112,36,-106,41),format='application/netcdf', store='true')
 
         is equivalent to:
-        http://myhost/mywcs?SERVICE=WCS&REQUEST=GetCoverage&IDENTIFIER=TuMYrRQ4&VERSION=1.1.0&BOUNDINGBOX=-180,-90,180,90&TIMESEQUENCE=[bb&FORMAT=application/netcdf
+        http://myhost/mywcs?SERVICE=WCS&REQUEST=GetCoverage&IDENTIFIER=TuMYrRQ4&VERSION=1.1.0&BOUNDINGBOX=-180,-90,180,90&TIMESEQUENCE=2792-06-01T00:00:00.0&FORMAT=application/netcdf
         
         if store = true, returns a coverages XML file
         if store = false, returns a multipart mime
         """
-        self.log.debug('WCS 1.1.0 DEBUG: Parameters passed to GetCoverage: identifier=%s, bbox=%s, time=%s, format=%s,        rangesubset=%s, gridbaseCRS=%s, gridtype=%s, gridCS=%s, gridorigin=%s, gridoffsets=%s, method=%s, other_arguments=%s'%(identifier, bbox, time, format, rangesubset, gridbaseCRS, gridtype, gridCS, gridorigin, gridoffsets, method, str(kwargs)))
+        if log.isEnabledFor(logging.DEBUG):
+            log.debug('WCS 1.1.0 DEBUG: Parameters passed to GetCoverage: identifier=%s, bbox=%s, time=%s, format=%s, rangesubset=%s, gridbaseCRS=%s, gridtype=%s, gridCS=%s, gridorigin=%s, gridoffsets=%s, method=%s, other_arguments=%s'%(identifier, bbox, time, format, rangesubset, gridbaseCRS, gridtype, gridCS, gridorigin, gridoffsets, method, str(kwargs)))
         
         
         if method == 'Get':
@@ -137,7 +150,7 @@ class WebCoverageService_1_1_0(WCSBase):
         request['identifier']=identifier
         #request['identifier'] = ','.join(identifier)
         if bbox:
-            request['boundingbox']=','.join([str(x) for x in bbox])
+            request['boundingbox']=','.join([repr(x) for x in bbox])
         if time:
             request['timesequence']=','.join(time)
         request['format']=format
@@ -166,33 +179,8 @@ class WebCoverageService_1_1_0(WCSBase):
         
         #encode and request
         data = urlencode(request)
-        #u = urlopen(base_url+data)
-        self.log.debug('WCS 1.1.0 DEBUG: base url of server: %s'%base_url)
-        self.log.debug('WCS 1.1.0 DEBUG: second part of URL: %s'%data)      
-        try:
-            u = urlopen(base_url + data) 
-            self.log.debug('WCS 1.1.0 DEBUG: called urlopen(base_url+data)')            
-        except:   
-            u = urlopen(base_url,data)
-            self.log.debug('WCS 1.1.0 DEBUG: called urlopen(base_url, data)')
-            
         
-        self.log.debug('WCS 1.1.0 DEBUG: GetCoverage request made: %s'%u.url)
-        self.log.debug('WCS 1.1.0 DEBUG: Headers returned: %s'%str(u.headers))
-                
-        # check for service exceptions, and return
-        if 'content-type' in u.info().keys():      
-            if u.info()['content-type'] == 'text/xml':          
-                #going to have to read the xml to see if it's an exception report.
-                #wrap the url stram in a extended StringIO object so it's re-readable
-                u=RereadableURL(u)      
-                se_xml= u.read()
-                se_tree = etree.fromstring(se_xml)
-                serviceException=se_tree.find('{http://www.opengis.net/ows}Exception')
-                if serviceException is not None:
-                    raise ServiceException, \
-                    str(serviceException.text).strip()
-                u.seek(0)
+        u=openURL(base_url, data, method, self.cookies)
         return u
         
         
@@ -222,8 +210,13 @@ class ServiceIdentification(object):
     def __init__(self,elem):        
         self.service="WCS"
         self.version="1.1.0"
-        self.title = elem.find('{http://www.opengis.net/ows}Title').text
-        self.abstract=elem.find('{http://www.opengis.net/ows}Abstract').text
+        self.title=testXMLValue(elem.find('{http://www.opengis.net/ows}Title'))
+        if self.title is None:  #may have used the wcs ows namespace:
+            self.title=testXMLValue(elem.find('{http://www.opengis.net/wcs/1.1/ows}Title'))
+        
+        self.abstract=testXMLValue(elem.find('{http://www.opengis.net/ows}Abstract'))
+        if self.abstract is None:#may have used the wcs ows namespace:
+            self.abstract=testXMLValue(elem.find('{http://www.opengis.net/wcs/1.1/ows}Abstract'))
         if elem.find('{http://www.opengis.net/ows}Abstract') is not None:
             self.abstract=elem.find('{http://www.opengis.net/ows}Abstract').text
         else:
@@ -325,7 +318,7 @@ class ContentMetadata(object):
                 if kw is not None:
                     self.keywords.append(kw.text)
             
-        
+        self.boundingBox=None #needed for iContentMetadata harmonisation
         self.boundingBoxWGS84 = None
         b = elem.find('{http://www.opengis.net/ows}WGS84BoundingBox')
         if b is not None:
@@ -350,18 +343,22 @@ class ContentMetadata(object):
                     self.boundingboxes.append(boundingBox)
                 except:
                      pass
+
+        #others not used but needed for iContentMetadata harmonisation
+        self.styles=None
+        self.crsOptions=None
                 
         #SupportedCRS
         self.supportedCRS=[]
         for crs in elem.findall('{http://www.opengis.net/wcs/1.1}SupportedCRS'):
-            self.supportedCRS.append(crs.text)
+            self.supportedCRS.append(Crs(crs.text))
+            
             
         #SupportedFormats         
         self.supportedFormats=[]
         for format in elem.findall('{http://www.opengis.net/wcs/1.1}SupportedFormat'):
             self.supportedFormats.append(format.text)
             
-        
     #grid is either a gml:Grid or a gml:RectifiedGrid if supplied as part of the DescribeCoverage response.
     def _getGrid(self):
         grid=None
