@@ -14,6 +14,9 @@ import StringIO
 import random
 from urllib import urlencode
 from urllib2 import urlopen
+
+from owslib.util import OrderedDict
+
 from owslib.etree import etree
 from owslib import fes
 from owslib import util
@@ -22,7 +25,7 @@ from owslib.iso import MD_Metadata
 from owslib.fgdc import Metadata
 from owslib.dif import DIF
 from owslib.namespaces import Namespaces
-from owslib.util import cleanup_namespaces, bind_url
+from owslib.util import cleanup_namespaces, bind_url, add_namespaces
 
 # default variables
 outputformat = 'application/xml'
@@ -234,7 +237,7 @@ class CatalogueServiceWeb:
             self.results['nextrecord'] = int(util.testXMLValue(val, True))
     
             # process list of matching records
-            self.records = {}
+            self.records = OrderedDict()
 
             self._parserecords(outputschema, esn)
 
@@ -270,7 +273,7 @@ class CatalogueServiceWeb:
  
         if self.exceptionreport is None:
             self.results = {}
-            self.records = {}
+            self.records = OrderedDict()
             self._parserecords(outputschema, esn)
 
     def getrecords2(self, constraints=[], sortby=None, typenames='csw:Record', esn='summary', outputschema=namespaces['csw'], format=outputformat, startposition=0, maxrecords=10, cql=None, xml=None, resulttype='results'):
@@ -334,7 +337,7 @@ class CatalogueServiceWeb:
                     etree.SubElement(node2, util.nspath_eval('csw:CqlText', namespaces)).text = cql
                 
             if sortby is not None and isinstance(sortby, fes.SortBy):
-                node1.append(sortby)
+                node1.append(sortby.toXML())
 
             self.request = node0
 
@@ -359,7 +362,7 @@ class CatalogueServiceWeb:
                 self.results['nextrecord'] = None
 
             # process list of matching records
-            self.records = {}
+            self.records = OrderedDict()
 
             self._parserecords(outputschema, esn)
 
@@ -472,6 +475,13 @@ class CatalogueServiceWeb:
                 self._parsetransactionsummary()
                 self._parseinsertresult()
 
+    def get_operation_by_name(self, name):
+        """Return a named operation"""
+        for item in self.operations:
+            if item.name.lower() == name.lower():
+                return item
+        raise KeyError, "No operation named %s" % name
+
     def getService_urls(self, service_string=None):
         """
 
@@ -577,6 +587,15 @@ class CatalogueServiceWeb:
             self.response = urlopen(self.request, timeout=self.timeout).read()
         else:
             self.request = cleanup_namespaces(self.request)
+            # Add any namespaces used in the "typeNames" attribute of the
+            # csw:Query element to the query's xml namespaces.
+            for query in self.request.findall(util.nspath_eval('csw:Query', namespaces)):
+                ns = query.get("typeNames", None)
+                if ns is not None:
+                    # Pull out "gmd" from something like "gmd:MD_Metadata"
+                    ns = ns.split(":")[0]
+                    self.request = add_namespaces(self.request, ns)
+
             self.request = util.xml2string(etree.tostring(self.request))
 
             self.response = util.http_post(self.url, self.request, self.lang, self.timeout)
@@ -664,7 +683,7 @@ class CswRecord(object):
         val = record.find(util.nspath_eval('dc:relation', namespaces))
         self.relation = util.testXMLValue(val)
 
-        val = record.find(util.nspath_eval('dc:temporal', namespaces))
+        val = record.find(util.nspath_eval('dct:temporal', namespaces))
         self.temporal = util.testXMLValue(val)
 
         self.uris = []  # list of dicts
