@@ -21,19 +21,7 @@ from urllib import urlencode
 from urllib2 import urlopen
 
 import logging
-
-try:
-    hdlr = logging.FileHandler('/tmp/owslibwfs.log')
-except:
-    import tempfile
-    f=tempfile.NamedTemporaryFile(prefix='owslib.wfs-', delete=False)
-    hdlr = logging.FileHandler(f.name)
-
-log = logging.getLogger(__name__)
-formatter = logging.Formatter('%(asctime)s %(levelname)s %(message)s')
-hdlr.setFormatter(formatter)
-log.addHandler(hdlr)
-log.setLevel(logging.DEBUG)
+from owslib.util import log
 
 n = Namespaces()
 WFS_NAMESPACE = n.get_namespace("wfs20")
@@ -65,9 +53,6 @@ class WebFeatureService_2_0_0(WebFeatureService_):
         """
         obj=object.__new__(self)
         obj.__init__(url, version, xml, parse_remote_metadata)
-        self.log = logging.getLogger()
-        consoleh  = logging.StreamHandler()
-        self.log.addHandler(consoleh)    
         return obj
     
     def __getitem__(self,name):
@@ -80,7 +65,8 @@ class WebFeatureService_2_0_0(WebFeatureService_):
     
     def __init__(self, url,  version, xml=None, parse_remote_metadata=False):
         """Initialize."""
-        log.debug('building WFS %s'%url)
+        if log.isEnabledFor(logging.DEBUG):
+            log.debug('building WFS %s'%url)
         self.url = url
         self.version = version
         self._capabilities = None
@@ -133,12 +119,12 @@ class WebFeatureService_2_0_0(WebFeatureService_):
         self.exceptions = [f.text for f \
                 in self._capabilities.findall('Capability/Exception/Format')]
       
-    def getcapabilities(self):
+    def getcapabilities(self, timeout=30):
         """Request and return capabilities document from the WFS as a 
         file-like object.
         NOTE: this is effectively redundant now"""
         reader = WFSCapabilitiesReader(self.version)
-        return urlopen(reader.capabilities_url(self.url))
+        return urlopen(reader.capabilities_url(self.url), timeout=timeout)
     
     def items(self):
         '''supports dict-like items() access'''
@@ -149,7 +135,7 @@ class WebFeatureService_2_0_0(WebFeatureService_):
     
     def getfeature(self, typename=None, filter=None, bbox=None, featureid=None,
                    featureversion=None, propertyname=None, maxfeatures=None,storedQueryID=None, storedQueryParams={},
-                   method='Get'):
+                   method='Get', timeout=30, outputFormat=None):
         """Request and return feature data as a file-like object.
         #TODO: NOTE: have changed property name from ['*'] to None - check the use of this in WFS 2.0
         Parameters
@@ -170,6 +156,10 @@ class WebFeatureService_2_0_0(WebFeatureService_):
             Maximum number of features to be returned.
         method : string
             Qualified name of the HTTP DCP method to use.
+        timeout : number
+            A timeout value (in seconds) for the request.
+        outputFormat: string (optional)
+            Requested response format of the request.
 
         There are 3 different modes of use
 
@@ -182,18 +172,18 @@ class WebFeatureService_2_0_0(WebFeatureService_):
         if typename and type(typename) == type(""):
             typename = [typename]
         if method.upper() == "GET":
-            (url) = self.getGETGetFeatureRequest(typename, filter, bbox, featureid, 
-                    featureversion, propertyname, maxfeatures,storedQueryID, storedQueryParams)
-            log.debug('GetFeature WFS GET url %s'% url)
+            (url) = self.getGETGetFeatureRequest(typename, filter, bbox, featureid,
+                                                 featureversion, propertyname,
+                                                 maxfeatures, storedQueryID,
+                                                 storedQueryParams, outputFormat)
+            if log.isEnabledFor(logging.DEBUG):
+                log.debug('GetFeature WFS GET url %s'% url)
         else:
             (url,data) = self.getPOSTGetFeatureRequest()
 
 
-
-        if method == 'Post':
-            u = urlopen(base_url, data=data)
-        else:
-            u = urlopen(url)
+        # If method is 'Post', data will be None here
+        u = urlopen(url, data, timeout)
         
         # check for service exceptions, rewrap, and return
         # We're going to assume that anything with a content-length > 32k
@@ -209,12 +199,18 @@ class WebFeatureService_2_0_0(WebFeatureService_):
         if length < 32000:
             if not have_read:
                 data = u.read()
-            tree = etree.fromstring(data)
-            if tree.tag == "{%s}ServiceExceptionReport" % OGC_NAMESPACE:
-                se = tree.find(nspath('ServiceException', OGC_NAMESPACE))
-                raise ServiceException, str(se.text).strip()
 
-            return StringIO(data)
+            try:
+                tree = etree.fromstring(data)
+            except BaseException:
+                # Not XML
+                return StringIO(data)
+            else:
+                if tree.tag == "{%s}ServiceExceptionReport" % OGC_NAMESPACE:
+                    se = tree.find(nspath('ServiceException', OGC_NAMESPACE))
+                    raise ServiceException(str(se.text).strip())
+                else:
+                    return StringIO(data)
         else:
             if have_read:
                 return StringIO(data)
@@ -236,12 +232,12 @@ class WebFeatureService_2_0_0(WebFeatureService_):
         if kwargs:
             for kw in kwargs.keys():
                 request[kw]=str(kwargs[kw])
-        data=urlencode(request)
-        u = urlopen(base_url + data)
+        encoded_request=urlencode(request)
+        u = urlopen(base_url + encoded_request)
         return u.read()
         
         
-    def _getStoredQueries(self):
+    def _getStoredQueries(self, timeout=30):
         ''' gets descriptions of the stored queries available on the server '''
         sqs=[]
         #This method makes two calls to the WFS - one ListStoredQueries, and one DescribeStoredQueries. The information is then
@@ -251,8 +247,8 @@ class WebFeatureService_2_0_0(WebFeatureService_):
         #first make the ListStoredQueries response and save the results in a dictionary if form {storedqueryid:(title, returnfeaturetype)}
         base_url = self.getOperationByName('ListStoredQueries').methods[method]['url']
         request = {'service': 'WFS', 'version': self.version, 'request': 'ListStoredQueries'}
-        data = urlencode(request)
-        u = urlopen(base_url + data)
+        encoded_request = urlencode(request)
+        u = urlopen(base_url + encoded_request, timeout=timeout)
         tree=etree.fromstring(u.read())
         base_url = self.getOperationByName('ListStoredQueries').methods[method]['url']
         tempdict={}       
@@ -269,8 +265,8 @@ class WebFeatureService_2_0_0(WebFeatureService_):
         #then make the DescribeStoredQueries request and get the rest of the information about the stored queries 
         base_url = self.getOperationByName('DescribeStoredQueries').methods[method]['url']
         request = {'service': 'WFS', 'version': self.version, 'request': 'DescribeStoredQueries'}
-        data = urlencode(request)
-        u = urlopen(base_url + data)
+        encoded_request = urlencode(request)
+        u = urlopen(base_url + encoded_request, timeout=timeout)
         tree=etree.fromstring(u.read())
         tempdict2={} 
         for sqelem in tree[:]:
@@ -320,7 +316,7 @@ class ContentMetadata:
     Implements IMetadata.
     """
 
-    def __init__(self, elem, parent, parse_remote_metadata=False):
+    def __init__(self, elem, parent, parse_remote_metadata=False, timeout=30):
         """."""
         self.id = elem.find(nspath('Name',ns=WFS_NAMESPACE)).text
         self.title = elem.find(nspath('Title',ns=WFS_NAMESPACE)).text
@@ -365,6 +361,7 @@ class ContentMetadata:
         #others not used but needed for iContentMetadata harmonisation
         self.styles=None
         self.timepositions=None
+        self.defaulttimeposition=None
 
         # MetadataURLs
         self.metadataUrls = []
@@ -377,7 +374,7 @@ class ContentMetadata:
 
             if metadataUrl['url'] is not None and parse_remote_metadata:  # download URL
                 try:
-                    content = urllib2.urlopen(metadataUrl['url'])
+                    content = urllib2.urlopen(metadataUrl['url'], timeout=timeout)
                     doc = etree.parse(content)
                     try:  # FGDC
                         metadataUrl['metadata'] = Metadata(doc)
@@ -417,7 +414,7 @@ class WFSCapabilitiesReader(object):
         urlqs = urlencode(tuple(qs))
         return service_url.split('?')[0] + '?' + urlqs
 
-    def read(self, url):
+    def read(self, url, timeout=30):
         """Get and parse a WFS capabilities document, returning an
         instance of WFSCapabilitiesInfoset
 
@@ -425,9 +422,11 @@ class WFSCapabilitiesReader(object):
         ----------
         url : string
             The URL to the WFS capabilities document.
+        timeout : number
+            A timeout value (in seconds) for the request.
         """
         request = self.capabilities_url(url)
-        u = urlopen(request)
+        u = urlopen(request, timeout=timeout)
         return etree.fromstring(u.read())
 
     def readString(self, st):
