@@ -2,10 +2,10 @@
 # ============================================================================= 
 # Copyright (c) 2009 Tom Kralidis
 #
-# Authors : Tom Kralidis <tomkralidis@hotmail.com>
+# Authors : Tom Kralidis <tomkralidis@gmail.com>
 #           Angelos Tzotsos <tzotsos@gmail.com>
 #
-# Contact email: tomkralidis@hotmail.com
+# Contact email: tomkralidis@gmail.com
 # =============================================================================
 
 """ ISO metadata parser """
@@ -295,13 +295,13 @@ class MD_DataIdentification(object):
 
             mdkw['thesaurus'] = {}
 
-            val = i.find(util.nspath_eval('gmd:thesaurusName/gmd:CI_Citation/gmd:title/gco:CharacterString', namespaces))
+            val = i.find(util.nspath_eval('gmd:MD_Keywords/gmd:thesaurusName/gmd:CI_Citation/gmd:title/gco:CharacterString', namespaces))
             mdkw['thesaurus']['title'] = util.testXMLValue(val)
 
-            val = i.find(util.nspath_eval('gmd:thesaurusName/gmd:CI_Citation/gmd:date/gmd:CI_Date/gmd:date/gco:Date', namespaces))
+            val = i.find(util.nspath_eval('gmd:MD_Keywords/gmd:thesaurusName/gmd:CI_Citation/gmd:date/gmd:CI_Date/gmd:date/gco:Date', namespaces))
             mdkw['thesaurus']['date'] = util.testXMLValue(val)
 
-            val = i.find(util.nspath_eval('gmd:thesaurusName/gmd:CI_Citation/gmd:date/gmd:CI_Date/gmd:dateType/gmd:CI_DateTypeCode', namespaces))
+            val = i.find(util.nspath_eval('gmd:MD_Keywords/gmd:thesaurusName/gmd:CI_Citation/gmd:date/gmd:CI_Date/gmd:dateType/gmd:CI_DateTypeCode', namespaces))
             mdkw['thesaurus']['datetype'] = util.testXMLValue(val)
 
             mdkw['keywords'] = []
@@ -343,11 +343,14 @@ class MD_DataIdentification(object):
 
             if val2 is None:
                 val2 = extent.find(util.nspath_eval('gmd:EX_Extent/gmd:temporalElement/gmd:EX_TemporalExtent/gmd:extent/gml:TimePeriod/gml:beginPosition', namespaces))
+                if val2 is None:
+                    val2 = extent.find(util.nspath_eval('gmd:EX_Extent/gmd:temporalElement/gmd:EX_TemporalExtent/gmd:extent/gml32:TimePeriod/gml32:beginPosition', namespaces))
                 self.temporalextent_start = util.testXMLValue(val2)
 
             if val3 is None:
-                self.temporalextent_end = []
                 val3 = extent.find(util.nspath_eval('gmd:EX_Extent/gmd:temporalElement/gmd:EX_TemporalExtent/gmd:extent/gml:TimePeriod/gml:endPosition', namespaces))
+                if val3 is None:
+                    val3 = extent.find(util.nspath_eval('gmd:EX_Extent/gmd:temporalElement/gmd:EX_TemporalExtent/gmd:extent/gml32:TimePeriod/gml32:endPosition', namespaces))
                 self.temporalextent_end = util.testXMLValue(val3)
 
 class MD_Distributor(object):        
