@@ -53,9 +53,8 @@ class SensorObservationService_1_0_0(object):
             self._capabilities = reader.read(self.url)
 
         # Avoid building metadata if the response is an Exception
-        se = self._capabilities.find(nspath_eval('ows:ExceptionReport', namespaces))
-        if se is not None: 
-            raise ows.ExceptionReport(se) 
+        if self._capabilities.tag == nspath_eval("ows:ExceptionReport", namespaces):
+            raise ows.ExceptionReport(self._capabilities)
 
         # build metadata objects
         self._build_metadata()
@@ -147,7 +146,7 @@ class SensorObservationService_1_0_0(object):
         request['offering'] = ','.join(offerings)
 
         assert isinstance(observedProperties, list) and len(observedProperties) > 0
-        request['observedproperty'] = ','.join(observedProperties)
+        request['observedProperty'] = ','.join(observedProperties)
 
         assert isinstance(responseFormat, str)
         request['responseFormat'] = responseFormat
@@ -168,6 +167,8 @@ class SensorObservationService_1_0_0(object):
             tr = etree.fromstring(response)
             if tr.tag == nspath_eval("ows:ExceptionReport", namespaces):
                 raise ows.ExceptionReport(tr)
+            else:
+                return response                
         except ows.ExceptionReport:
             raise
         except BaseException:
@@ -262,8 +263,8 @@ class SosCapabilitiesReader(object):
             qs.append(('service', 'SOS'))
         if 'request' not in params:
             qs.append(('request', 'GetCapabilities'))
-        if 'version' not in params:
-            qs.append(('version', self.version))
+        if 'acceptVersions' not in params:
+            qs.append(('acceptVersions', self.version))
 
         urlqs = urlencode(tuple(qs))
         return service_url.split('?')[0] + '?' + urlqs
@@ -274,7 +275,7 @@ class SosCapabilitiesReader(object):
             elementtree instance
 
             service_url is the base url, to which is appended the service,
-            version, and request parameters
+            acceptVersions, and request parameters
         """
         getcaprequest = self.capabilities_url(service_url)
         spliturl=getcaprequest.split('?')
