@@ -2,9 +2,9 @@
 # =============================================================================
 # Copyright (c) 2008 Tom Kralidis
 #
-# Authors : Tom Kralidis <tomkralidis@hotmail.com>
+# Authors : Tom Kralidis <tomkralidis@gmail.com>
 #
-# Contact email: tomkralidis@hotmail.com
+# Contact email: tomkralidis@gmail.com
 # =============================================================================
 
 import sys
@@ -12,6 +12,7 @@ from dateutil import parser
 from datetime import datetime
 import pytz
 from owslib.etree import etree
+from owslib.namespaces import Namespaces
 import urlparse, urllib2
 from urllib2 import urlopen, HTTPError, Request
 from urllib2 import HTTPPasswordMgrWithDefaultRealm
@@ -20,6 +21,7 @@ from StringIO import StringIO
 import cgi
 from urllib import urlencode
 import re
+from copy import deepcopy
 
 
 """
@@ -114,7 +116,7 @@ def xml_to_dict(root, prefix=None, depth=1, diction=None):
 
     return ret
 
-def openURL(url_base, data, method='Get', cookies=None, username=None, password=None):
+def openURL(url_base, data, method='Get', cookies=None, username=None, password=None, timeout=30):
     ''' function to open urls - wrapper around urllib2.urlopen but with additional checks for OGC service exceptions and url formatting, also handles cookies and simple user password authentication'''
     url_base.strip() 
     lastchar = url_base[-1]
@@ -152,14 +154,14 @@ def openURL(url_base, data, method='Get', cookies=None, username=None, password=
             req=Request(url_base + data)
         if cookies is not None:
             req.add_header('Cookie', cookies)
-        u = openit(req)
+        u = openit(req, timeout=timeout)
     except HTTPError, e: #Some servers may set the http header to 400 if returning an OGC service exception or 401 if unauthorised.
         if e.code in [400, 401]:
             raise ServiceException, e.read()
         else:
             raise e
     # check for service exceptions without the http header set
-    if u.info()['Content-Type'] in ['text/xml', 'application/xml']:          
+    if ((u.info().has_key('Content-Type')) and (u.info()['Content-Type'] in ['text/xml', 'application/xml'])):          
         #just in case 400 headers were not set, going to have to read the xml to see if it's an exception report.
         #wrap the url stram in a extended StringIO object so it's re-readable
         u=RereadableURL(u)      
@@ -216,6 +218,46 @@ def cleanup_namespaces(element):
     else:
         return etree.fromstring(etree.tostring(element))
 
+
+def add_namespaces(root, ns_keys):
+    if isinstance(ns_keys, basestring):
+        ns_keys = [ns_keys]
+
+    namespaces = Namespaces()
+
+    ns_keys = map(lambda x: (x, namespaces.get_namespace(x)), ns_keys)
+
+    if etree.__name__ != 'lxml.etree':
+        # We can just add more namespaces when not using lxml.
+        # We can't re-add an existing namespaces.  Get a list of current
+        # namespaces in use
+        existing_namespaces = set()
+        for elem in root.getiterator():
+            if elem.tag[0] == "{":
+                uri, tag = elem.tag[1:].split("}")
+                existing_namespaces.add(namespaces.get_namespace_from_url(uri))
+        for key, link in ns_keys:
+            if link is not None and key not in existing_namespaces:
+                root.set("xmlns:%s" % key, link)
+        return root
+    else:
+        # lxml does not support setting xmlns attributes
+        # Update the elements nsmap with new namespaces
+        new_map = root.nsmap
+        for key, link in ns_keys:
+            if link is not None:
+                new_map[key] = link
+        # Recreate the root element with updated nsmap
+        new_root = etree.Element(root.tag, nsmap=new_map)
+        # Carry over attributes
+        for a, v in root.items():
+            new_root.set(a, v)
+        # Carry over children
+        for child in root:
+            new_root.append(deepcopy(child))
+        return new_root
+
+
 def testXMLValue(val, attrib=False):
     """
 
@@ -251,9 +293,7 @@ def testXMLAttribute(element, attribute):
 
     """
     if element is not None:
-        attrib = element.get(attribute)
-        if attrib is not None:
-            return attrib.strip()
+        return element.get(attribute)
 
     return None
 
@@ -444,3 +484,23 @@ def bind_url(url):
         elif url.find('&', -1) == -1: # like http://host/wms?foo=bar
             binder = '&'
     return '%s%s' % (url, binder)
+
+import logging
+# Null logging handler
+try:
+    # Python 2.7
+    NullHandler = logging.NullHandler
+except AttributeError:
+    # Python < 2.7
+    class NullHandler(logging.Handler):
+        def emit(self, record):
+            pass
+log = logging.getLogger('owslib')
+log.addHandler(NullHandler())
+
+# OrderedDict
+try:  # 2.7
+    from collections import OrderedDict
+except:  # 2.6
+    from ordereddict import OrderedDict
+
