@@ -18,6 +18,7 @@ Currently supports only version 1.1.1 of the WMS protocol.
 import cgi
 import urllib2
 from urllib import urlencode
+import warnings
 from etree import etree
 from .util import openURL, testXMLValue, extract_xml_list
 from fgdc import Metadata
@@ -120,7 +121,7 @@ class WebMapService(object):
                 cm = ContentMetadata(elem, parent=parent_metadata, index=index+1, parse_remote_metadata=parse_remote_metadata)
                 if cm.id:
                     if cm.id in self.contents:
-                        raise KeyError('Content metadata for layer "%s" already exists' % cm.id)
+                        warnings.warn('Content metadata for layer "%s" already exists. Using child layer' % cm.id)
                     self.contents[cm.id] = cm
                 gather_layers(elem, cm)
         gather_layers(caps, None)
@@ -188,15 +189,15 @@ class WebMapService(object):
         
         Example
         -------
-            >>> img = wms.getmap(layers=['global_mosaic'],
-            ...                  styles=['visual'],
-            ...                  srs='EPSG:4326', 
-            ...                  bbox=(-112,36,-106,41),
-            ...                  format='image/jpeg',
-            ...                  size=(300,250),
-            ...                  transparent=True,
-            ...                  )
-            >>> out = open('example.jpg', 'wb')
+            >>> wms = WebMapService('http://giswebservices.massgis.state.ma.us/geoserver/wms', version='1.1.1')
+            >>> img = wms.getmap(layers=['massgis:GISDATA.SHORELINES_ARC'],\
+                                 styles=[''],\
+                                 srs='EPSG:4326',\
+                                 bbox=(-70.8, 42, -70, 42.8),\
+                                 size=(300, 300),\
+                                 format='image/jpeg',\
+                                 transparent=True)
+            >>> out = open('example.jpg.jpg', 'wb')
             >>> out.write(img.read())
             >>> out.close()
 
@@ -311,7 +312,7 @@ class ContentMetadata:
 
     Implements IContentMetadata.
     """
-    def __init__(self, elem, parent=None, index=0, parse_remote_metadata=False):
+    def __init__(self, elem, parent=None, index=0, parse_remote_metadata=False, timeout=30):
         if elem.tag != 'Layer':
             raise ValueError('%s should be a Layer' % (elem,))
         
@@ -446,10 +447,12 @@ class ContentMetadata:
 
         # timepositions - times for which data is available.
         self.timepositions=None
+        self.defaulttimeposition = None
         for extent in elem.findall('Extent'):
             if extent.attrib.get("name").lower() =='time':
                 if extent.text:
                     self.timepositions=extent.text.split(',')
+                    self.defaulttimeposition = extent.attrib.get("default")
                     break
                 
         # Elevations - available vertical levels
@@ -471,7 +474,7 @@ class ContentMetadata:
 
             if metadataUrl['url'] is not None and parse_remote_metadata:  # download URL
                 try:
-                    content = urllib2.urlopen(metadataUrl['url'])
+                    content = urllib2.urlopen(metadataUrl['url'], timeout=timeout)
                     doc = etree.parse(content)
                     if metadataUrl['type'] is not None:
                         if metadataUrl['type'] == 'FGDC':
