@@ -104,7 +104,7 @@ GML_NAMESPACE = n.get_namespace("gml")
 DRAW_NAMESPACE = n.get_namespace("draw")
 
 GML_SCHEMA_LOCATION = "http://schemas.opengis.net/gml/3.1.1/base/feature.xsd"
-DRAW_SCHEMA_LOCATION = 'http://cida.usgs.gov/qa/climate/derivative/xsd/draw.xsd'
+DRAW_SCHEMA_LOCATION = 'http://cida.usgs.gov/climate/derivative/xsd/draw.xsd'
 WPS_DEFAULT_SCHEMA_LOCATION = 'http://schemas.opengis.net/wps/1.0.0/wpsExecute_request.xsd'
 WPS_DEFAULT_VERSION = '1.0.0'
 
@@ -463,6 +463,8 @@ class WPSExecution():
         self.process = None
         self.serviceInstance = None
         self.status = None
+        self.percentCompleted = 0
+        self.statusMessage = None
         self.errors = []
         self.statusLocation = None
         self.dataInputs=[]
@@ -548,12 +550,20 @@ class WPSExecution():
             responseFormElement = etree.SubElement(root, nspath_eval('wps:ResponseForm', namespaces))
             responseDocumentElement = etree.SubElement(responseFormElement, nspath_eval('wps:ResponseDocument', namespaces), 
                                                        attrib={'storeExecuteResponse':'true', 'status':'true'} )
-            outputElement = etree.SubElement(responseDocumentElement, nspath_eval('wps:Output', namespaces), 
-                                                       attrib={'asReference':'true'} )
-            outputIdentifierElement = etree.SubElement(outputElement, nspath_eval('ows:Identifier', namespaces)).text = output
-                  
-
+            if isinstance(output, str):
+                self._add_output(responseDocumentElement, output, asReference=True)
+            elif isinstance(output, list):
+                for (identifier,as_reference) in output:
+                    self._add_output(responseDocumentElement, identifier, asReference=as_reference)
+            else:
+                raise Exception('output parameter is neither string nor list. output=%s' % output)
         return root
+
+    def _add_output(self, element, identifier, asReference=False):
+        outputElement = etree.SubElement(element, nspath_eval('wps:Output', namespaces), 
+                                                       attrib={'asReference':str(asReference).lower()} )
+        outputIdentifierElement = etree.SubElement(outputElement, nspath_eval('ows:Identifier', namespaces)).text = identifier
+                  
                 
     # wait for 60 seconds by default
     def checkStatus(self, url=None, response=None, sleepSecs=60):
@@ -613,8 +623,8 @@ class WPSExecution():
         
     def getOutput(self, filepath=None):
         """
-        Method to write the output of a WPS process to a file: 
-        either retrieve the referenced file from the server, or write out the content of response embedded output.
+        Method to write the outputs of a WPS process to a file: 
+        either retrieves the referenced files from the server, or writes out the content of response embedded output.
         
         filepath: optional path to the output file, otherwise a file will be created in the local directory with the name assigned by the server, 
                   or default name 'wps.out' for embedded output.
@@ -624,25 +634,14 @@ class WPSExecution():
             content = ''
             for output in self.processOutputs:
                 
+                output_content = output.retrieveData(self.username, self.password)
+                
                 # ExecuteResponse contains reference to server-side output
-                if output.reference is not None:
-                    # a) 'http://cida.usgs.gov/climate/gdp/process/RetrieveResultServlet?id=1318528582026OUTPUT.601bb3d0-547f-4eab-8642-7c7d2834459e'
-                    # b) 'http://rsg.pml.ac.uk/wps/wpsoutputs/outputImage-11294Bd6l2a.tif'
-                    url = output.reference
-                    print 'Output URL=%s' % url
-                    if '?' in url:
-                        spliturl=url.split('?')
-                        u = openURL(spliturl[0], spliturl[1], method='Get', username = self.username, password = self.password)
-                        # extract output filepath from URL query string
-                        if filepath is None:
-                            filepath = spliturl[1].split('=')[1]
-                    else:
-                        u = openURL(url, '', method='Get', username = self.username, password = self.password)
-                        # extract output filepath from base URL
-                        if filepath is None:
-                            filepath = url.split('/')[-1]
-                    content = content + u.read()
-                 
+                if output_content is not "":
+                    content = content + output_content
+                    if filepath is None:
+                        filepath = output.fileName
+                    
                 # ExecuteResponse contain embedded output   
                 if len(output.data)>0:
                     if filepath is None:
@@ -705,6 +704,8 @@ class WPSExecution():
             
         # print status, errors
         print 'Execution status=%s' % self.status
+        print 'Percent completed=%s' % self.percentCompleted
+        print 'Status message=%s' % self.statusMessage
         for error in self.errors:
             dump(error)
 
@@ -717,7 +718,7 @@ class WPSExecution():
         if self.status is None:
             self.status = "Exception"
             
-        for exceptionEl in root.findall( nspath('Exception', ns=getNamespace(root)) ):
+        for exceptionEl in root.findall( nspath('Exception', ns=namespaces['ows']) ):
             self.errors.append( WPSException(exceptionEl) )
 
 
@@ -747,11 +748,19 @@ class WPSExecution():
         # </ns0:Status>
         statusEl = root.find( nspath('Status/*', ns=wpsns) )
         self.status = statusEl.tag.split('}')[1]
+        # get progress info
+        try:
+            percentCompleted = int(statusEl.get('percentCompleted'))
+            self.percentCompleted = percentCompleted
+        except:
+            pass
+        # get status message
+        self.statusMessage = statusEl.text
         # exceptions ?
         for element in statusEl:
             if element.tag.endswith('ExceptionReport'):
                 self._parseExceptionReport(element)
-        
+
         self.process = Process(root.find(nspath('Process', ns=wpsns)), verbose=self.verbose)
         
         #<wps:DataInputs xmlns:wps="http://www.opengis.net/wps/1.0.0"
@@ -960,6 +969,8 @@ class Output(InputOutput):
         self.reference = None
         self.mimeType = None
         self.data = []
+        self.fileName = None
+        self.filePath = None
         
         # extract wps namespace from outputElement itself
         wpsns = getNamespace(outputElement)
@@ -971,8 +982,8 @@ class Output(InputOutput):
             self.reference = referenceElement.get('href')
             self.mimeType = referenceElement.get('mimeType')
        
-        # <LiteralData>
-        self._parseLiteralData(outputElement, 'LiteralData')
+        # <LiteralOutput>
+        self._parseLiteralData(outputElement, 'LiteralOutput')
         
         # <ComplexData> or <ComplexOutput>
         self._parseComplexData(outputElement, 'ComplexOutput')
@@ -1020,6 +1031,67 @@ class Output(InputOutput):
                     self.data.append(complexDataElement.text.strip())
                 for child in complexDataElement:
                     self.data.append(etree.tostring(child))
+            literalDataElement = dataElement.find( nspath('LiteralData', ns=wpsns) )
+            if literalDataElement is not None:
+                self.dataType = literalDataElement.get('dataType')
+                if literalDataElement.text is not None and literalDataElement.text.strip() is not '':
+                    self.data.append(literalDataElement.text.strip())
+                    
+    def retrieveData(self, username=None, password=None):
+        """
+        Method to retrieve data from server-side reference: 
+        returns "" if the reference is not known.
+        
+        username, password: credentials to access the remote WPS server 
+        """
+        
+        url = self.reference
+        if url is None: 
+            return ""
+        
+        # a) 'http://cida.usgs.gov/climate/gdp/process/RetrieveResultServlet?id=1318528582026OUTPUT.601bb3d0-547f-4eab-8642-7c7d2834459e'
+        # b) 'http://rsg.pml.ac.uk/wps/wpsoutputs/outputImage-11294Bd6l2a.tif'
+        print 'Output URL=%s' % url
+        if '?' in url:
+            spliturl=url.split('?')
+            u = openURL(spliturl[0], spliturl[1], method='Get', username = username, password = password)
+            # extract output filepath from URL query string
+            self.fileName = spliturl[1].split('=')[1]
+        else:
+            u = openURL(url, '', method='Get', username = username, password = password)
+            # extract output filepath from base URL
+            self.fileName = url.split('/')[-1]
+                
+        return u.read()      
+
+                    
+    def writeToDisk(self, path=None, username=None, password=None):
+        """
+        Method to write an output of a WPS process to disk: 
+        it either retrieves the referenced file from the server, or write out the content of response embedded output.
+        
+        filepath: optional path to the output file, otherwise a file will be created in the local directory with the name assigned by the server,
+        username, password: credentials to access the remote WPS server
+        """ 
+        
+        # Check if ExecuteResponse contains reference to server-side output    
+        content = self.retrieveData(username, password)
+         
+        # ExecuteResponse contain embedded output   
+        if content is "" and len(self.data)>0:
+            self.fileName = self.identifier
+            for data in self.data:
+                content = content + data
+                    
+        # write out content
+        if content is not "":
+            if self.fileName == "":
+                self.fileName = self.identifier
+            self.filePath = path + self.fileName
+            out = open(self.filePath, 'wb')
+            out.write(content)
+            out.close()
+            print 'Output written to file: %s' %self.filePath
                 
                     
 class WPSException:
@@ -1207,7 +1279,7 @@ class GMLMultiPolygonFeatureCollection(FeatureCollection):
                         xmlns:draw="gov.usgs.cida.gdp.draw" xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance"
                         xmlns:ows="http://www.opengis.net/ows" xmlns:gml="http://www.opengis.net/gml"
                         xmlns:xlink="http://www.w3.org/1999/xlink"
-                        xsi:schemaLocation="gov.usgs.cida.gdp.draw http://cida.usgs.gov/qa/climate/derivative/xsd/draw.xsd">
+                        xsi:schemaLocation="gov.usgs.cida.gdp.draw http://cida.usgs.gov/climate/derivative/xsd/draw.xsd">
                         <gml:box gml:id="box.1">
                             <gml:the_geom>
                                 <gml:MultiPolygon srsDimension="2"
